import logging


class Logger:
    """This logger will last the length of the process.  Only one is created. Further instantiation simply returns
        the existing instance.  Thus only the initial caller gets to name the log.info"""

    log = None

    @staticmethod
    def create_logger(name=None, log_file=None):
        if not name:
            name = 'tlx'
        logger = logging.getLogger(name)
        logger.setLevel(logging.DEBUG)

        if log_file:
            # create file handler which logs even debug messages
            fh = logging.FileHandler(log_file)
            fh.setLevel(logging.DEBUG)

        # create console handler
        ch = logging.StreamHandler()
        ch.setLevel(logging.DEBUG)

        # create formatter and add it to the handler
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')

        ch.setFormatter(formatter)
        logger.addHandler(ch)

        if log_file:  # repeat the above if file output
            fh.setFormatter(formatter)
            logger.addHandler(fh)

        # Do some logging
        logger.debug('Logger initialised ({})'.format(name))

        Logger.log = logger

    def __init__(self, name=None, log_file=None):
        if not Logger.log:
            Logger.create_logger(name, log_file)


def get_log():
    logger = Logger()
    return logger.log



def ensure_http_success(func):
    """May be used as a method decorator. Raises a `RuntimeError` if the input function does not return a
    successful HTTPStatusCode.  `KeyError` if the input function is not a Boto3 API call.

    Args:
        func (func):  This should be a boto3 API call.

    Returns:
        func
    """
    def wrapper(*args, **kwargs):
        response = func(*args, **kwargs)
        if not 200 <= response['ResponseMetadata']['HTTPStatusCode'] < 300:
            raise RuntimeError("API call failed! :", response['ResponseMetadata'])
        return response
    return wrapper



import json
from decimal import Decimal
from collections import defaultdict


def _pull_values(item):
    return {k: _set_types(v) for k, v in item.items()}


_func_map = defaultdict(
    lambda: lambda x: x, {
        'M': _pull_values,
        'N': Decimal,
        'L': lambda values: [_set_types(d) for d in values]
    }
)


def _set_types(v):
    v_key = list(v.keys())[0]
    returned_value = list(v.values())[0]  # Always one from dump
    return _func_map[v_key](returned_value)


def load_data(scan_dump_file, table):
    """
        Loads the results of a scan opperation into a table.
    """
    with open(scan_dump_file, 'r') as f:
        scan_dump = json.load(f)

    with table.batch_writer() as batch:
        for item in scan_dump['Items']:
            batch.put_item(
                Item=_pull_values(item),
            )



import boto3
import logging
from botocore.credentials import DeferredRefreshableCredentials
from botocore.exceptions import ClientError


class Session(boto3.session.Session):
    """Returns an AWS session instance.  If no parameters are provided, keys from `.aws/credentials` 'default' profile are used.

        Kwargs (all optional):
            region (str): An AWS region
            profile (str): A profile with API keys as configured in ~/.aws/credentials file. Not to be used with other parmeters.
                If your role needs an MFA token you will be prompted for input.
            role (str): AWS Arn of the role the user is assuming. If None, the users identity is used.
            mfa_serial (str): AWS Arn of the users Multi-Factor authentication device.
            mfa_token (str): 6 digit Multi-Factor Authentication code.

        Example:
            session = Session()
            s3client = session.client('s3')
    """

    def __init__(self, region=None, profile=None, role=None, mfa_serial=None, mfa_token=None):
        if profile and role:
            raise AttributeError("Either a profile should be used OR a role assumed. Not both.")

        if (mfa_serial and not mfa_token) or (mfa_token and not mfa_serial):
            raise AttributeError("If using MFA, provide both a serial and a token.")

        params = {
            'region_name': region,
        }

        if role:
            creds = _assume_role(role, mfa_serial, mfa_token)
            params.update({
                'aws_access_key_id': creds['AccessKeyId'],
                'aws_secret_access_key': creds['SecretAccessKey'],
                'aws_session_token': creds['SessionToken'],
            })
        elif profile:
            params['profile_name'] = profile
        try:
            boto3.session.Session.__init__(self, **{k: v for k, v in params.items() if v})
        except ClientError as e:
            logging.error("Do the API keys in `~/.aws/credentials` match those required? \n{}".format(e))

    def get_session_creds(self):
        creds = self.get_credentials()

        if isinstance(creds, DeferredRefreshableCredentials):
            # prompt for MFA token
            creds = creds.get_frozen_credentials()

        return creds


def _assume_role(role, mfa_serial, mfa_token):
    # TODO: Check for HTTP fail
    params = {
        "RoleArn": role,
        "RoleSessionName": 'ecs-deploy-session',
        "DurationSeconds": 3600,
        "SerialNumber": mfa_serial,
        "TokenCode": mfa_token
    }
    return boto3.client('sts').assume_role(**{k: v for k, v in params.items() if v})['Credentials']



#!/usr/bin/env python

from __future__ import print_function
import click
import sys
from tlx.util import Session


@click.command(context_settings=dict(max_content_width=120))
@click.option('--profile', '-p', default='default', help="A profile defined in `~/.aws/credentials`.  If it requires an MFA token a prompt will be given")
@click.option('--quiet', default=False, is_flag=True, help='If using the outputs directly, see --help.')
def main(profile, quiet):
    """
        GET AWS CREDS (gac):

        Gets temporary AWS session from a profile.  Allows you to export into your shell and run tools expecting the AWS standard environment variables. Namely:

        \b
        - AWS_ACCESS_KEY_ID
        - AWS_SECRET_ACCESS_KEY
        - AWS_SESSION_TOKEN

        Your aws credentials file (default: `~/.aws/credentials`) should be configured like so:
        \b
        Configure `credentials` like so:
            [profilename]
            region = us-east-2
            role_arn = arn:aws:iam::************:role/Demo-StackCreator
            mfa_serial = arn:aws:iam::************:mfa/IAM-F.Name
            source_profile = default

        Returns: commands to copy to your shell terminal

        Consider appending the following to your `.bashrc` to have the environment variables set automatically.
        `gac(){ for i in "$( gac "$@" --quiet)"; do eval "${i}"; done;}`

    """
    try:
        session = Session(profile=profile)
        creds = session.get_session_creds()
    except Exception as e:
        print("{}: {}".format(type(e).__name__, e))
        sys.exit(1)

    if not quiet:
        print("Keys and token for profile: '{profile}'".format(profile=profile))
        print("Paste the following into your shell:\n")

    print("export AWS_ACCESS_KEY_ID={}".format(creds.access_key))
    print("export AWS_SECRET_ACCESS_KEY={}".format(creds.secret_key))
    print("export AWS_SESSION_TOKEN={}".format(creds.token))



from setuptools import setup, find_packages
import os

here = os.path.dirname(os.path.realpath(__file__))


def get_install_requires():
    with open(os.path.join(here, 'requirements.txt'), 'r') as f:
        return f.read().splitlines()


def get_version():
    with open(os.path.join(here, 'version.txt'), 'r') as f:
        return f.readline().split()[0]  # Filter out other junk


setup(
    name='tlx',
    version=get_version(),
    description='Frequently used utilities and code.',
    url='https://github.com/eL0ck/tlx',
    author='eL0ck',
    author_email='tpj800@gmail.com',
    license='Apache',
    python_requires='>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, <4',
    packages=find_packages(),
    install_requires=get_install_requires(),
    entry_points={
        'console_scripts': [
            'get-aws-creds=tlx.util.cli_apps.get_aws_creds:main',
            'dbp=tlx.dynamodb.cli_apps.dynamodb_batch_prepare:main',
            'dynamo-batch-prepare=tlx.dynamodb.cli_apps.dynamodb_batch_prepare:main',
        ],
    },
    data_files=[
        'version.txt',
        'requirements.txt',
    ],
    zip_safe=False,
)



from __future__ import print_function
import json
import click


@click.command(context_settings=dict(max_content_width=120, help_option_names=['-h', '--help']))
@click.option('--dump-file', '-d', required=True, type=click.File('rb'), help="File dumped from dynamodb with.")
@click.option('--table-name', '-n', required=True, help="Table to send data to. Table must exist and key schema must match.  Use `aws dynamodb describe-table --table-name <TableName>`")
def main(dump_file, table_name):
    """
        DYNAMO BATCH PREPARE (dbp)

        \b
        `dynamodb-batch-prepare --help`
        OR
        `dbp --help`

        Takes the output of a `scan` operation such as: `aws dynamodb scan --table-name <TableName>`
        and formats for use by:

        `aws dynamodb batch-write-item --request-items file://output.json`
    """

    scan_data = json.load(dump_file)

    table_name = 'StageData'
    items = scan_data['Items']

    batch_request = {
        table_name: [{"PutRequest": {"Item": item}} for item in items]
    }

    print(json.dumps(batch_request, indent=4, separators=(',', ': '), sort_keys=True))
